%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%This is a example for the calculation of the modified Control Effective%
%Fluxes (mCEF)                                                          % 
%                                                                       %
%Corresponding author: Prof. Hiroyuki Kurata                            %
%kurata@bio.kyutech.ac.jp                                               %
%Kyushu Institute of Technology, Iizuka, Fukuoka, Japan                 %
%2008.08                                                                %  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function mcef1(em,ni,cellfunc,eta)
% em    elementary mode matrix
% ni;   the reaction serial number of the substrate uptake reaction in EM
% matrix
% cellfunc is the row vector including the reaction serial numbers of the cellular
% functions in EM matrix, for example, the Biomass and the ATP drain reactions
% eta; column vector fo the relative enzyme activities of reactions
% for mutant versus wild type

[a1,a2]=size(em);%%%%%%%%%%%%%%%%%%%%%%%%%
% a1: the number ofreactions; 
% a2: the number ofEMs;

% Step 1: normalization of EM matrix;
for i=1:a2
    if em(ni,i)~=0
        nem(:,i)=em(:,i)/em(ni,i);
    else
        nem(:,i)=em(:,i);
    end
end
% nem is the normalized matrix

[ctemp,cfnumber]=size(cellfunc);% cfnumber is the total number of cellular function

% Step 2: Calculate the mCEF for wild type
mcefw=zeros(a1,1);% mCEF is modified CEF of wild typy
emefficiencyw=zeros(a2,cfnumber); % emefficiency is the efficiency of EM
for j=1:1:a2
    for i=1:1:cfnumber
        stemp=0;
        for k=1:1:a1
            if k~=ni
                stemp=stemp+abs(nem(k,j));
            end
        end
    emefficiencyw(j,i)=nem(cellfunc(1,i),j)/stemp;
    end
end
%emefficiencyw 

for j=1:1:a1
    for i=1:1:cfnumber
        stemp1=0;
        stemp2=0;
        for k=1:1:a2
            stemp1=stemp1+emefficiencyw(k,i);
            stemp2=stemp2+emefficiencyw(k,i)*abs(nem(j,k));
        end
        mcefw(j,1)=mcefw(j,1)+stemp2/stemp1/max(nem(cellfunc(1,i),:));
    end
end


% Step 3: Calculate mCEF for mutant
eda=ones(a2,1);% special parameter for EMs

neda=0;% neda is the total number of genetic modified reactions
for i=1:a1
    if eta(i,1)~=1
        neda=neda+1;
        nedanumber(neda,1)=i;% nedanumber shows which reaction is modified
    end
end

for i=1:neda
    for j=1:a2
       if em(nedanumber(i,1),j)~=0
           eda(j,1)=eta(nedanumber(i,1),1)*eda(j,1);
       end
    end
end
    
mcef=zeros(a1,1);% mCEF is modified CEF
emefficiency=zeros(a2,cfnumber); % emefficiency is the efficiency of EM
for j=1:1:a2
    for i=1:1:cfnumber
        stemp=0;
        for k=1:1:a1
            if k~=ni
                stemp=stemp+abs(nem(k,j))*eta(k,1);
            end
        end
    emefficiency(j,i)=nem(cellfunc(1,i),j)*eda(j,1)/stemp;
    end
end
%emefficiency

for j=1:1:a1
    for i=1:1:cfnumber
        stemp1=0;
        stemp2=0;
        for k=1:1:a2
            stemp1=stemp1+emefficiency(k,i);
            stemp2=stemp2+emefficiency(k,i)*abs(nem(j,k))*eta(j,1);
        end
        mcef(j,1)=mcef(j,1)+stemp2/stemp1/max(nem(cellfunc(1,i),:));
    end
end

for i=1:1:a1
    if mcefw(i,1)~=0
    rge(i,1)=mcef(i,1)/mcefw(i,1);
    else
        rge(i,1)=1;
    end
end

disp('Relative gene expression is')
rge

save mcefrge.mat rge
    